/*
	PROforma, PROGS Font & Raster Manager

	LaserJet printer driver

	(c) PROGS, Professional & Graphical Software
	started 04 October 1995
	28.10.1995 problem with x and y skip fixed
	02.11.1995 made alloc/release output channel much safer
*/

#include "mem_h"
#include "driv_pcl_h"

Bbox PrintableArea={
	short2pt(571.2), short2pt(817.68),	/* size */
	short2pt(12), short2pt(12)			/* origin */
	};

char DefaultOutputDevice[IO_MAXFULLNAME]="pard";

Error Define(char *name, char *value)
{
	double x,y;

	if (STRSameCD(name,"DEFAULT-DEVICE"))
	{
		STRCopySafe(value,DefaultOutputDevice,IO_MAXFULLNAME);
		return ERR_OK;
	}
	if (STRSameCD(name,"PRINTABLE-AREA-SIZE"))
	{
		STRFormatGet(value,NULL,"%f %f",&x,&y);
		PrintableArea.xsiz=double2pt(x);
		PrintableArea.ysiz=double2pt(y);
		return ERR_OK;
	}
	if (STRSameCD(name,"PRINTABLE-AREA-ORIGIN"))
	{
		STRFormatGet(value,NULL,"%f %f",&x,&y);
		PrintableArea.xorg=double2pt(x);
		PrintableArea.yorg=double2pt(y);
		return ERR_OK;
	}
	return ERR_IPAR;
}

Error InitInit(Gstate gstate, char *device, PBbox *area)
{
	Error err;
	DriverInfo *di;
	DRIVdriver *prt=gstate->printer;

	err=MEMAllocate(sizeof(DriverInfo),&di);
	if (err) return err;

	di->type=LEVEL5+MINIMARGIN+UNIT+COPIES;/* this is a LaserJet4 driver */
	di->copies=1;					/* start with one copy a page */

	if (device && *device)
		err=IOOpen(device,OPEN_OVER,&di->channel);
	else
		err=IOOpen(DefaultOutputDevice,OPEN_OVER,&di->channel);
	if (err) { MEMRelease(di); return err; };

	gstate->PrinterExtra=di;
	gstate->todev.x = pt_one * prt->xdpi / 72;
	gstate->todev.y = pt_one * prt->ydpi / 72;
	gstate->ppp.x = pt_one * 72 / prt->xdpi;
	gstate->ppp.y = pt_one * 72 / prt->ydpi;

	area->xsiz=pt2rshort(fixmul(PrintableArea.xsiz,gstate->todev.x));
	area->ysiz=pt2rshort(fixmul(PrintableArea.ysiz,gstate->todev.y));
	area->xorg=pt2rshort(fixmul(PrintableArea.xorg,gstate->todev.x));
	area->yorg=pt2rshort(fixmul(PrintableArea.yorg,gstate->todev.y));

	return err;
}

Error InitDone(Gstate gstate)
{
	DriverInfo *di=gstate->PrinterExtra;
	Size linc;

	gstate->bitmap->BufferSize(gstate->pagebbox.xsiz,2,&linc);

	/* complete the initialisation of the DriverInfo */
	di->out_line =(char *)gstate->raster->base;
	di->prev_line=di->out_line+linc;
	di->alt_line =di->prev_line+linc;
	gstate->raster->base=(unsigned char *)(di->alt_line+linc);

	di->xskip=pt2rshort(gstate->pagebbox.xporg-fixmul(PrintableArea.xorg,
							   gstate->todev.x));
	di->yskip=pt2rshort(gstate->pagebbox.yporg-fixmul(PrintableArea.yorg,
							   gstate->todev.y));

	return ERR_OK;
}

Error BufferSize(Gstate gstate, int xsiz, int ysiz, Size *size)
{
	return gstate->bitmap->BufferSize(xsiz,ysiz+6,size);
}

Error CleanUp(Gstate gstate)
{
	DriverInfo *di=gstate->PrinterExtra;

	if (di)
	{
		IOClose(di->channel);
		MEMRelease(gstate->PrinterExtra);
	}
	return ERR_OK;
}

Error ColourNearest(Gstate gstate, pt *colour, int *result)
{
	*result=0;						/* assume white */
	if (*colour>pt_hundred/2)
		*result=1;					/* make it black */
	return ERR_OK;
}

Error ColourDelta(Gstate gstate, pt *colour, pt *delta)
{
	*delta=*colour;
	if (*colour>pt_hundred/2)
		*delta-=pt_hundred;
	return ERR_OK;
}

static Error nimp() 	{ return ERR_NIMP; }
static Error nothing()	{ return ERR_OK; }

Error InitDriver();

Error SetCopies(Gstate, int count);
Error SetDisplayMode(Gstate, int mode);
Error WindowMove(Gstate, int dx, int dy);
Error BufferUpdate(Gstate);
Error InitPage(Gstate);
Error ShowPage(Gstate);
Error StopPage(Gstate);
Error ColourAdjust(Gstate, pt *);
Error GetFullPageBbox(Gstate, Bbox *box);
Error DefaultDevice(Gstate, char *device);

#define InitDriver		nothing
#define WindowMove		nothing
#define ColourAdjust	nothing
#define SetDisplayMode	nimp
#define GetFullPageBbox nimp
#define DefaultDevice	nimp
#define BufferUpdate	(Error (*)(Gstate))nimp
#define BufferSave		(Error (*)(Gstate))nimp
#define BufferRestore	(Error (*)(Gstate))nimp
#define Handle			nimp

DRIVdriver dpi100={
	NULL, PF_PRINTERDRIVER,
	"HP LaserJet 4 100dpi", 	/* name */
	100,100,					/* xdpi, ydpi */
	{                           /* mode 4 bitmap */
		1, 1,					/* 1 plane, 1 pixel deep */
		BITMAP_NO_BYTEGROUPING, /* no byte grouping */
		BITMAP_NO_PLANES,		/* no planes */
	},

	InitDriver,
	Define,

	InitInit,
	BufferSize,
	InitDone,
	CleanUp,

	InitPage,
	ShowPage,
	StopPage,

	SetCopies,
	SetDisplayMode,
	WindowMove,

	BufferUpdate,
	BufferSave,
	BufferRestore,

	PF_COLOURSPACE_GRAYSHADE,
	{
		ColourAdjust,
		ColourNearest,
		ColourDelta
	},

	GetFullPageBbox,
	DefaultDevice,

	Handle
};

DRIVdriver dpi150={
	&dpi100, PF_PRINTERDRIVER,
	"HP LaserJet 4 150dpi", 	/* name */
	150,150,					/* xdpi, ydpi */
	{                           /* mode 4 bitmap */
		1, 1,					/* 1 plane, 1 pixel deep */
		BITMAP_NO_BYTEGROUPING, /* no byte grouping */
		BITMAP_NO_PLANES,		/* no planes */
	},

	InitDriver,
	Define,

	InitInit,
	BufferSize,
	InitDone,
	CleanUp,

	InitPage,
	ShowPage,
	StopPage,

	SetCopies,
	SetDisplayMode,
	WindowMove,

	BufferUpdate,
	BufferSave,
	BufferRestore,

	PF_COLOURSPACE_GRAYSHADE,
	{
		ColourAdjust,
		ColourNearest,
		ColourDelta
	},

	GetFullPageBbox,
	DefaultDevice,

	Handle
};

DRIVdriver dpi300={
	&dpi150, PF_PRINTERDRIVER,
	"HP LaserJet 4 300dpi", 	/* name */
	300,300,					/* xdpi, ydpi */
	{                           /* mode 4 bitmap */
		1, 1,					/* 1 plane, 1 pixel deep */
		BITMAP_NO_BYTEGROUPING, /* no byte grouping */
		BITMAP_NO_PLANES,		/* no planes */
	},

	InitDriver,
	Define,

	InitInit,
	BufferSize,
	InitDone,
	CleanUp,

	InitPage,
	ShowPage,
	StopPage,

	SetCopies,
	SetDisplayMode,
	WindowMove,

	BufferUpdate,
	BufferSave,
	BufferRestore,

	PF_COLOURSPACE_GRAYSHADE,
	{
		ColourAdjust,
		ColourNearest,
		ColourDelta
	},

	GetFullPageBbox,
	DefaultDevice,

	Handle
};

DRIVdriver init={
	&dpi300, PF_PRINTERDRIVER,
	"HP LaserJet 4 600dpi", 	/* name */
	600,600,					/* xdpi, ydpi */
	{                           /* mode 4 bitmap */
		1, 1,					/* 1 plane, 1 pixel deep */
		BITMAP_NO_BYTEGROUPING, /* no byte grouping */
		BITMAP_NO_PLANES,		/* no planes */
	},

	InitDriver,
	Define,

	InitInit,
	BufferSize,
	InitDone,
	CleanUp,

	InitPage,
	ShowPage,
	StopPage,

	SetCopies,
	SetDisplayMode,
	WindowMove,

	BufferUpdate,
	BufferSave,
	BufferRestore,

	PF_COLOURSPACE_GRAYSHADE,
	{
		ColourAdjust,
		ColourNearest,
		ColourDelta
	},

	GetFullPageBbox,
	DefaultDevice,

	Handle
};
